/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik.enginebased.afm;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import org.assertj.core.data.MapEntry;

import de.ozgcloud.eingang.common.formdata.Antragsteller;

public class AfmAntragstellerTestFactory {

	public static final String ANREDE = "Herr";
	public static final String FIRMA_NAME = "Firma X";
	public static final String VORNAME = "Max";
	public static final String NACHNAME = "Mustermann";
	public static final String GEBURTSNAME = "Maxi";
	public static final String GEBURTSDATUM = "01.01.1970";
	public static final String GEBURTSORT = "Sandhausen";
	public static final String EMAIL = "alsob@ne.ver";
	public static final String TELEFON = "+ 49 4621 9654";
	public static final String STRASSE = "Lachstrasse";
	public static final String HAUSNUMMER = "8484";
	public static final String PLZ = "12345";
	public static final String ORT = "Wohlfuehlhausen";

	public static final String POSTFACH_ID = UUID.randomUUID().toString();

	public static Antragsteller create() {
		return createBuilder().build();
	}

	public static Antragsteller createFirm() {
		return createBuilder()
				.firmaName(FIRMA_NAME)
				.geburtsname(null)
				.geburtsdatum(null)
				.geburtsort(null)
				.build();
	}

	public static Antragsteller.AntragstellerBuilder createBuilder() {
		return Antragsteller.builder()
				.anrede(ANREDE)
				.vorname(VORNAME)
				.nachname(NACHNAME)
				.geburtsname(GEBURTSNAME)
				.geburtsdatum(GEBURTSDATUM)
				.geburtsort(GEBURTSORT)
				.email(EMAIL)
				.telefon(TELEFON)
				.strasse(STRASSE)
				.hausnummer(HAUSNUMMER)
				.plz(PLZ)
				.ort(ORT)
				.postfachId(POSTFACH_ID);
	}

	public static Map<String, Object> createFormDataMap() {
		return createFormDataMap(AfmAntragstellerMapper.ANTRAGSTELLER);
	}

	public static Map<String, Object> createFormDataMap(String antragstellerKey) {
		return createMutableFormDataMap(antragstellerKey);
	}

	public static Map<String, Object> createMutableFormDataMap(String antragstellerKey) {
		var map = new HashMap<String, Object>();
		map.put(antragstellerKey, createPrivateAntragstelleMap());
		map.put(AfmAntragstellerMapper.POSTFACH_ID, POSTFACH_ID);
		return map;
	}

	@SafeVarargs
	public static Map<String, Object> createPrivateAntragstelleMap(MapEntry<String, Object>... additionalEntries) {
		var map = new HashMap<String, Object>();
		map.put(AfmAntragstellerMapper.ANREDE, ANREDE);
		map.put(AfmAntragstellerMapper.VORNAME, VORNAME);
		map.put(AfmAntragstellerMapper.NACHNAME, NACHNAME);
		map.put(AfmAntragstellerMapper.GEBURTSNAME, GEBURTSNAME);
		map.put(AfmAntragstellerMapper.GEBURTSDATUM, GEBURTSDATUM);
		map.put(AfmAntragstellerMapper.GEBURTSORT, GEBURTSORT);
		map.put(AfmAntragstellerMapper.EMAIL, EMAIL);
		map.put(AfmAntragstellerMapper.TELEFON, TELEFON);
		map.put(AfmAntragstellerMapper.STRASSE, STRASSE);
		map.put(AfmAntragstellerMapper.HAUSNUMMER, HAUSNUMMER);
		map.put(AfmAntragstellerMapper.PLZ, PLZ);
		map.put(AfmAntragstellerMapper.ORT, ORT);

		Arrays.stream(additionalEntries).forEach(entry -> map.put(entry.getKey(), entry.getValue()));

		return Collections.unmodifiableMap(map);
	}

	@SafeVarargs
	public static Map<String, Object> createFirmAntragstelleMap(MapEntry<String, Object>... additionalEntries) {
		var map = new HashMap<String, Object>();
		map.put(AfmAntragstellerMapper.FIELD_FIRMA_NAME, FIRMA_NAME);
		map.put(AfmAntragstellerMapper.FIELD_ANSPRECHPARTNER_ANREDE, ANREDE);
		map.put(AfmAntragstellerMapper.FIELD_ANSPRECHPARTNER_VORNAME, VORNAME);
		map.put(AfmAntragstellerMapper.FIELD_ANSPRECHPARTNER_NACHNAME, NACHNAME);
		map.put(AfmAntragstellerMapper.EMAIL, EMAIL);
		map.put(AfmAntragstellerMapper.TELEFON, TELEFON);
		map.put(AfmAntragstellerMapper.FIELD_FIRMA_STRASSE, STRASSE);
		map.put(AfmAntragstellerMapper.FIELD_FIRMA_HAUSNUMMER, HAUSNUMMER);
		map.put(AfmAntragstellerMapper.FIELD_FIRMA_PLZ, PLZ);
		map.put(AfmAntragstellerMapper.FIELD_FIRMA_STADT, ORT);

		Arrays.stream(additionalEntries).forEach(entry -> map.put(entry.getKey(), entry.getValue()));

		return Collections.unmodifiableMap(map);
	}
}