package de.ozgcloud.eingang.xta;

import java.util.Spliterators;
import java.util.stream.Stream;
import java.util.stream.StreamSupport;

import de.ozgcloud.eingang.common.vorgang.VorgangNummerSupplier;
import de.ozgcloud.eingang.xdomea.XdomeaMessageDataMapper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.ozgcloud.eingang.common.formdata.FormData;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;

@Service
@Log4j2
class XtaService {

	static final String XDOMEA_0201_MESSAGE_TYPE = "Geschaeftsgang.Geschaeftsgang.0201";

	static final String ALL_FIM_MESSAGES_PREFIX = "fim.";

	@Autowired
	private XtaRemoteService remoteService;
	@Autowired
	private XtaMessageMapper mapper;
	@Autowired
	private VorgangNummerSupplier vorgangNummerSupplier;
	@Autowired
	private XdomeaMessageDataMapper xdomeaMessageDataMapper;
	@Autowired
	private FimMessageMapper fimMessageMapper;
	@Autowired
	private XtaIncomingFilesMapper xtaIncomingFilesMapper;

	public Stream<FormData> getMessages() {
		return createXtaMessageStream().filter(this::isSupportedMessageType).map(this::getFormData);
	}

	Stream<XtaMessageMetaData> createXtaMessageStream() {
		var iterator = new XtaMessageMetadataRemoteIterator(remoteService);
		return StreamSupport.stream(Spliterators.spliteratorUnknownSize(iterator, 0), false);
	}

	boolean isSupportedMessageType(XtaMessageMetaData metaData) {
		if (StringUtils.startsWith(metaData.getMessageType(), ALL_FIM_MESSAGES_PREFIX)) {
			return true;
		}

		LOG.warn("Ignoring XTA-Message of type '{}'.", metaData.getMessageType());
		return false;
	}

	public FormData getFormData(@NonNull XtaMessageMetaData metaData) {
		var msg = remoteService.getMessage(metaData.getMessageId());
		var incomingFiles = xtaIncomingFilesMapper.toIncomingFiles(msg.getMessageFiles());

		// FIXME should be called based on messageType and not hard coded
		return fimMessageMapper.toFormData(metaData, vorgangNummerSupplier, incomingFiles);
	}

	public void acknowledgeReceive(@NonNull XtaMessageId messageId) {
		remoteService.close(messageId);
	}
}
